h1 {
	font-size: larger;
	margin: initial;
}

table.reftable {
	border: 1px solid black;
	border-collapse: collapse;
}

table.reftable tr td {
	border: 1px solid black;
}

td.back,
td.next {
	width: 50px;
}

dt {
	 font-weight: bold;
}

.fullwidth {
	width: 100%;
}

.left {
	text-align: left;
}

.center {
	text-align: center;
}

.right {
	text-align: right;
}

header nav a,
footer nav a,
nav li,
nav li a {
	padding: 3px;
	margin: 3px;
	user-select: none;
}

main {
	border-top: 1px solid gray;
	border-bottom: 1px solid gray;
}
